require_relative 'tag_cloud'
require_relative 'rss_builder'

module Contraption
  class Site
    def initialize location, posts, formats
      @location = location
      @formats = formats
      @posts = posts
    end

    def build_all_individuals
      @posts.each.to_a.each { |i| build_individual i }
    end

    def build_individual i
      post_location = @location.cd(post_path i)
      context = build i
      page = @formats.format(context, :page)
      post_location.write(i.filename('.html'), page)
    end

    def build_month_pages
      build_pages_by(:month) {|m| "#{m.year}/#{"%02d" % m.month}"}
    end

    def build_year_pages
      build_pages_by(:year) {|y| y.year.to_s}
    end

    def build_tag_pages
      build_pages_by(:tag) {|t| "tags/#{Tag.new(t).to_url}"}
    end

    def build_recent
      recent_posts = @posts.most_recent 8
      combined_posts = build recent_posts
      write_as_page combined_posts, @location, 'recent.html'
    end

    def build_landing
      content = @formats.format({most_recent: @posts.most_recent.first}, :landing_page)
      @location.write('index.html', content)
    end

    def build_tag_cloud
      tag_cloud = TagCloud.new @posts
      content = @formats.format({content: tag_cloud.to_s}, :tag_cloud)
      path = @location.cd 'tags'
      write_as_page content, path
    end

    def build_rss
      feed = RSSBuilder.new(@posts).to_rss
      @location.write 'rss', feed
    end

    def build_archive_navigation
      content = @posts.by_month
                      .keys
                      .sort_by{|m| m.year*12 + m.month}
                      .map{|m| %Q{<a class="indivisible" href="#{m.year}/#{"%02d" % m.month}">#{Date::ABBR_MONTHNAMES[m.month]} #{m.year}</a>}}
                      .join("\n")
      write_as_page content, @location, 'archive.html'
    end

    def root
      @location.path
    end

    def location
      @location
    end

    private
    def build collection
      Array(collection).map {|p| @formats.format p}.join
    end

    def build_pages_by type, &determine_path
      @posts.public_send("by_#{type}".to_sym).each_pair do |key, posts|
        path = @location.cd determine_path.call key
        combined_posts = build posts
        intermediate_page = @formats.format({content: combined_posts, key: key}, type.to_sym)
        write_as_page intermediate_page, path
      end
    end

    def post_path post
      post.publication_date.strftime('%Y') + '/' + post.publication_date.strftime('%m')
    end

    def write_as_page content, path, name='index.html'
      page = @formats.format(content, :page)
      path.write(name, page)
    end
  end
end
